% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-Chromatogram.R
\docType{methods}
\name{findChromPeaks,Chromatogram,CentWaveParam-method}
\alias{findChromPeaks,Chromatogram,CentWaveParam-method}
\title{centWave-based peak detection in purely chromatographic data}
\usage{
\S4method{findChromPeaks}{Chromatogram,CentWaveParam}(object, param, ...)
}
\arguments{
\item{object}{a \link{Chromatogram} or \link{Chromatograms} object.}

\item{param}{a \link{CentWaveParam} object specifying the settings for the
peak detection. See \code{\link[=peaksWithCentWave]{peaksWithCentWave()}} for the description of
arguments used for peak detection.}

\item{...}{currently ignored.}
}
\value{
If called on a \code{Chromatogram} object, the method returns a \code{matrix} with
the identified peaks. See \code{\link[=peaksWithCentWave]{peaksWithCentWave()}} for details on the matrix
content.
}
\description{
\code{findChromPeaks} on a \link{Chromatogram} or \link{Chromatograms} object with a
\link{CentWaveParam} parameter object performs centWave-based peak detection
on purely chromatographic data. See \link{centWave} for details on the method
and \link{CentWaveParam} for details on the parameter class.
Note that not all settings from the \code{CentWaveParam} will be used.
See \code{\link[=peaksWithCentWave]{peaksWithCentWave()}} for the arguments used for peak detection
on purely chromatographic data.
}
\examples{

od <- readMSData(system.file("cdf/KO/ko15.CDF", package = "faahKO"),
    mode = "onDisk")

## Extract chromatographic data for a small m/z range
chr <- chromatogram(od, mz = c(272.1, 272.3))[1, 1]

## Identify peaks with default settings
pks <- findChromPeaks(chr, CentWaveParam())
pks

## Plot the identified peaks
plot(chr, type = "h")
rect(xleft = pks[, "rtmin"], xright = pks[, "rtmax"],
    ybottom = rep(0, nrow(pks)), ytop = pks[, "maxo"], col = "#ff000020")

## Modify the settings
cwp <- CentWaveParam(snthresh = 5, peakwidth = c(10, 60))
pks <- findChromPeaks(chr, cwp)
pks

plot(chr, type = "h")
rect(xleft = pks[, "rtmin"], xright = pks[, "rtmax"],
    ybottom = rep(0, nrow(pks)), ytop = pks[, "maxo"], col = "#00ff0020")
}
\seealso{
\code{\link[=peaksWithCentWave]{peaksWithCentWave()}} for the downstream function and \link{centWave}
for details on the method.
}
\author{
Johannes Rainer
}
